@@ -130,7 +130,12 @@ Mail::Mail (LPDISPATCH mailitem) :
     m_needs_encrypt(false),
     m_moss_position(0),
     m_crypto_flags(0),
-    m_type(MSGTYPE_UNKNOWN)
+    m_cached_html_body(nullptr),
+    m_cached_plain_body(nullptr),
+    m_cached_recipients(nullptr),
+    m_type(MSGTYPE_UNKNOWN),
+    m_do_inline(false),
+    m_is_gsuite(false)
 {
   if (get_mail_for_item (mailitem))
     {
@@ -193,6 +198,11 @@ Mail::~Mail()
       log_oom_extra ("%s:%s: non crypto mail: %p destroyed",
                      SRCNAME, __func__, this);
     }
+  xfree (m_cached_html_body);
+  xfree (m_cached_plain_body);
+  for (int i = 0; m_cached_recipients && m_cached_recipients[i]; ++i)
+      xfree (m_cached_recipients[i]);
+  xfree (m_cached_recipients);
   gpgrt_lock_unlock (&dtor_lock);
 }
 
@@ -250,8 +260,8 @@ Mail::pre_process_message ()
                  SRCNAME, __func__);
       return 0;
     }
-  log_oom_extra ("%s:%s: GetBaseMessage OK.",
-                 SRCNAME, __func__);
+  log_oom_extra ("%s:%s: GetBaseMessage OK for %p.",
+                 SRCNAME, __func__, m_mailitem);
   /* Change the message class here. It is important that
      we change the message class in the before read event
      regardless if it is already set to one of GpgOL's message
@@ -733,6 +743,8 @@ Mail::decrypt_verify()
 {
   if (!is_crypto_mail())
     {
+      log_debug ("%s:%s: Decrypt Verify for non crypto mail: %p.",
+                 SRCNAME, __func__, m_mailitem);
       return 0;
     }
   if (m_needs_wipe)
@@ -995,46 +1007,63 @@ Mail::encrypt_sign ()
       int n_att_usable = count_usable_attachments (att_table);
       mapi_release_attach_table (att_table);
       /* Check for attachments if we have some abort. */
+
+      wchar_t *w_title = utf8_to_wchar (_(
+                              "GpgOL: Oops, G Suite Sync account detected"));
       if (n_att_usable)
         {
-          MessageBox (window,
+          wchar_t *msg = utf8_to_wchar (
                       _("G Suite Sync breaks outgoing crypto mails "
                         "with attachments.\nUsing crypto and attachments "
                         "with G Suite Sync is not supported.\n\n"
-                        "See: https://dev.gnupg.org/T3545 for details."),
-                      _("GpgOL: Oops, G Suite Sync account detected"),
-                      MB_ICONINFORMATION|MB_OK);
+                        "See: https://dev.gnupg.org/T3545 for details."));
+          MessageBoxW (window,
+                       msg,
+                       w_title,
+                       MB_ICONINFORMATION|MB_OK);
+          xfree (msg);
+          xfree (w_title);
           return -1;
         }
       if (flags == 2)
         {
-          MessageBox (window,
-                      _("G Suite Sync breaks outgoing signed mails.\n"
+          wchar_t *msg = utf8_to_wchar (
+                       _("G Suite Sync breaks outgoing signed mails.\n"
                         "Ensuring mail integrity (signing) with G Suite Sync "
                         "is not supported.\n\n"
-                        "See: https://dev.gnupg.org/T3545 for details."),
-                      _("GpgOL: Oops, G Suite Sync account detected"),
-                      MB_ICONINFORMATION|MB_OK);
+                        "See: https://dev.gnupg.org/T3545 for details."));
+          MessageBoxW (window,
+                       msg,
+                       w_title,
+                       MB_ICONINFORMATION|MB_OK);
+          xfree (msg);
+          xfree (w_title);
           return -1;
         }
       if (flags == 3)
         {
-          if(MessageBox (window,
+          wchar_t *msg = utf8_to_wchar (
                         _("G Suite Sync breaks outgoing signed mails.\n"
                           "Ensuring mail integrity (signing) with G Suite Sync "
                           "is not supported.\n\n"
                           "See: https://dev.gnupg.org/T3545 for details.\n\n"
-                          "Do you want to only encrypt the message?"),
-                        _("GpgOL: Oops, G Suite Sync account detected"),
-                        MB_ICONINFORMATION|MB_YESNO) != IDYES)
+                          "Do you want to only encrypt the message?"));
+          if(MessageBoxW (window,
+                          msg,
+                          w_title,
+                          MB_ICONINFORMATION|MB_YESNO) != IDYES)
             {
+              xfree (msg);
+              xfree (w_title);
               return -1;
             }
           else
             {
               flags = 1;
             }
+          xfree (msg);
         }
+      xfree (w_title);
     }
 
   m_do_inline = m_is_gsuite ? true : opt.inline_pgp;
@@ -1117,24 +1146,30 @@ Mail::update_oom_data ()
   LPDISPATCH sender = NULL;
   log_debug ("%s:%s", SRCNAME, __func__);
 
-  /* Update the body format. */
-  m_is_html_alternative = get_oom_int (m_mailitem, "BodyFormat") > 1;
-
-  /* Store the body. It was not obvious for me (aheinecke) how
-     to access this through MAPI. */
-  if (m_is_html_alternative)
+  if (!is_crypto_mail())
     {
-      log_debug ("%s:%s: Is html alternative mail.", SRCNAME, __func__);
-      const char * html_body = get_oom_string (m_mailitem, "HTMLBody");
-      if (html_body)
+      /* Update the body format. */
+      m_is_html_alternative = get_oom_int (m_mailitem, "BodyFormat") > 1;
+
+      /* Store the body. It was not obvious for me (aheinecke) how
+         to access this through MAPI. */
+      if (m_is_html_alternative)
         {
-          m_html_body = html_body;
+          log_debug ("%s:%s: Is html alternative mail.", SRCNAME, __func__);
+          xfree (m_cached_html_body);
+          m_cached_html_body = get_oom_string (m_mailitem, "HTMLBody");
         }
+      xfree (m_cached_plain_body);
+      m_cached_plain_body = get_oom_string (m_mailitem, "Body");
+
+      for (int i = 0; m_cached_recipients && m_cached_recipients[i]; ++i)
+          xfree (m_cached_recipients[i]);
+      xfree (m_cached_recipients);
+      m_cached_recipients = get_recipients ();
     }
   /* For some reason outlook may store the recipient address
      in the send using account field. If we have SMTP we prefer
      the SenderEmailAddress string. */
-
   if (is_crypto_mail ())
     {
       /* This is the case where we are reading a mail and not composing.
@@ -2337,10 +2372,20 @@ Mail::is_html_alternative () const
   return m_is_html_alternative;
 }
 
-const std::string &
-Mail::get_cached_html_body () const
+char *
+Mail::take_cached_html_body ()
 {
-  return m_html_body;
+  char *ret = m_cached_html_body;
+  m_cached_html_body = nullptr;
+  return ret;
+}
+
+char *
+Mail::take_cached_plain_body ()
+{
+  char *ret = m_cached_plain_body;
+  m_cached_plain_body = nullptr;
+  return ret;
 }
 
 int
@@ -2360,3 +2405,29 @@ Mail::needs_encrypt() const
 {
   return m_needs_encrypt;
 }
+
+char **
+Mail::take_cached_recipients()
+{
+  char **ret = m_cached_recipients;
+  m_cached_recipients = nullptr;
+  return ret;
+}
+
+void
+Mail::append_to_inline_body (const std::string &data)
+{
+  m_inline_body += data;
+}
+
+int
+Mail::inline_body_to_body()
+{
+  if (m_inline_body.empty())
+    {
+      return 0;
+    }
+  int ret = put_oom_string (m_mailitem, "Body", m_inline_body.c_str ());
+  m_inline_body = std::string();
+  return ret;
+}
