@@ -304,8 +304,20 @@ public:
     Only valid if update_oom_data was called before. */
   bool is_html_alternative () const;
 
-  /** Get the html body. It is updated in update_oom_data. */
-  const std::string & get_cached_html_body () const;
+  /** Get the html body. It is updated in update_oom_data.
+      Caller takes ownership of the string and has to free it.
+  */
+  char *take_cached_html_body ();
+
+  /** Get the plain body. It is updated in update_oom_data.
+      Caller takes ownership of the string and has to free it.
+  */
+  char *take_cached_plain_body ();
+
+  /** Get the cached recipients. It is updated in update_oom_data.
+      Caller takes ownership of the list and has to free it.
+  */
+  char **take_cached_recipients ();
 
   /** Returns 1 if the mail was encrypted, 2 if signed, 3 if both.
       Only valid after decrypt_verify.
@@ -328,6 +340,13 @@ public:
 
   /** Check if the mail should be encrypted "inline" */
   bool should_inline_crypt () const {return m_do_inline;}
+
+  /** Append data to a cached inline body. Helper to do this
+     on MAPI level and later add it through OOM */
+  void append_to_inline_body (const std::string &data);
+
+  /** Set the inline body as OOM body property. */
+  int inline_body_to_body ();
 private:
   void update_categories ();
   void update_body ();
@@ -349,7 +368,9 @@ private:
   int m_moss_position; /* The number of the original message attachment. */
   int m_crypto_flags;
   std::string m_sender;
-  std::string m_html_body; /* Cached html body. */
+  char *m_cached_html_body; /* Cached html body. */
+  char *m_cached_plain_body; /* Cached plain body. */
+  char **m_cached_recipients;
   msgtype_t m_type; /* Our messagetype as set in mapi */
   std::shared_ptr <ParseController> m_parser;
   GpgME::VerificationResult m_verify_result;
@@ -360,5 +381,6 @@ private:
   std::string m_orig_body;
   bool m_do_inline;
   bool m_is_gsuite; /* Are we on a gsuite account */
+  std::string m_inline_body;
 };
 #endif // MAIL_H
