@@ -23,6 +23,7 @@
 #include "eventsink.h"
 #include "eventsinks.h"
 #include "mymapi.h"
+#include "mymapitags.h"
 #include "oomhelp.h"
 #include "ocidl.h"
 #include "windowmessages.h"
@@ -189,6 +190,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__, m_mail);
           if (!m_mail->is_crypto_mail())
             {
+              log_debug ("%s:%s: Non crypto mail %p opened. Updating sigstatus.",
+                         SRCNAME, __func__, m_mail);
               /* Ensure that no wrong sigstatus is shown */
               CloseHandle(CreateThread (NULL, 0, delayed_invalidate_ui, (LPVOID) this, 0,
                                         NULL));
@@ -334,8 +337,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
                          SRCNAME, __func__, m_mail);
           if (!m_mail->needs_crypto ())
             {
-             log_debug ("%s:%s: No crypto neccessary. Passing send for unencrypted %p",
-                        SRCNAME, __func__, m_mail);
+             log_debug ("%s:%s: No crypto neccessary. Passing send for %p obj %p",
+                        SRCNAME, __func__, m_mail, m_object);
              break;
             }
 
@@ -347,6 +350,7 @@ EVENT_SINK_INVOKE(MailItemEvents)
            }
           m_mail->update_oom_data ();
           m_mail->set_needs_encrypt (true);
+
           invoke_oom_method (m_object, "Save", NULL);
           if (m_mail->crypto_successful ())
             {
@@ -358,6 +362,14 @@ EVENT_SINK_INVOKE(MailItemEvents)
                 {
                   log_debug ("%s:%s: Body found after encryption %p.",
                             SRCNAME, __func__, m_object);
+                  if (m_mail->should_inline_crypt ())
+                    {
+                      if (m_mail->inline_body_to_body ())
+                        {
+                          log_debug ("%s:%s: Inline body to body failed %p.",
+                                    SRCNAME, __func__, m_object);
+                        }
+                    }
                   const auto body = m_mail->get_body();
                   if (body.size() > 10 && !strncmp (body.c_str(), "-----BEGIN", 10))
                     {
@@ -366,29 +378,77 @@ EVENT_SINK_INVOKE(MailItemEvents)
                       break;
                     }
 
-                  if (!m_mail->wipe (true))
+                  if (m_mail->wipe (true))
                     {
-                      log_debug ("%s:%s: Wipe succeded. %p.",
-                                 SRCNAME, __func__, m_object);
-
-                      log_debug ("%s:%s: Passing send event for message %p.",
+                      log_debug ("%s:%s: Cancel send for %p.",
                                 SRCNAME, __func__, m_object);
+                      wchar_t *title = utf8_to_wchar (_("GpgOL: Encryption not possible!"));
+                      wchar_t *msg = utf8_to_wchar (_(
+                      "Outlook returned an error when trying to send the encrypted mail.\n\n"
+                      "Please restart Outlook and try again.\n\n"
+                      "If it still fails consider using an encrypted attachment or\n"
+                      "switching to PGP/Inline in GpgOL's options."));
+                      MessageBoxW (get_active_hwnd(), msg, title,
+                                   MB_ICONERROR | MB_OK);
+                      xfree (msg);
+                      xfree (title);
+                      *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+                      return S_OK;
+                    }
+                    {
+                      /* Now we adress T3656 if Outlooks internal S/MIME is somehow
+                       * mixed in (even if it is enabled and then disabled) it might
+                       * cause strange behavior in that it sends the plain message
+                       * and not the encrypted message. Tests have shown that we can
+                       * bypass that by calling submit message on our base
+                       * message.
+                       *
+                       * We do this conditionally as our other way of using OOM
+                       * to send is proven to work and we don't want to mess
+                       * with it.
+                       */
+                      // Get the Message class.
+                      HRESULT hr;
+                      LPSPropValue propval = NULL;
+
+                      // It's important we use the _not_ base message here.
+                      LPMESSAGE message = get_oom_message (m_object);
+                      hr = HrGetOneProp ((LPMAPIPROP)message, PR_MESSAGE_CLASS_A, &propval);
+                      gpgol_release (message);
+                      if (FAILED (hr))
+                        {
+                          log_error ("%s:%s: HrGetOneProp() failed: hr=%#lx\n",
+                                     SRCNAME, __func__, hr);
+                          gpgol_release (message);
+                          break;
+                        }
+                      if (propval->Value.lpszA && !strstr (propval->Value.lpszA, "GpgOL"))
+                        {
+                          // Does not have a message class by us.
+                          log_debug ("%s:%s: Message %p - No GpgOL Message class after encryption.",
+                                    SRCNAME, __func__, m_object);
+                          log_debug ("%s:%s: Message %p - Activating T3656 Workaround",
+                                    SRCNAME, __func__, m_object);
+                          message = get_oom_base_message (m_object);
+                          // It's important we use the _base_ message here.
+                          mapi_save_changes (message, KEEP_OPEN_READWRITE | FORCE_SAVE);
+                          message->SubmitMessage(0);
+                          gpgol_release (message);
+
+                          // Cancel send
+                          *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
+
+                          // Close the composer and trigger unloads
+                          CloseHandle(CreateThread (NULL, 0, close_mail, (LPVOID) m_mail, 0,
+                                                    NULL));
+                        }
+                      MAPIFreeBuffer (propval);
+                    }
+
+                  if (*(parms->rgvarg[0].pboolVal) == VARIANT_TRUE)
+                    {
                       break;
                     }
-                  log_debug ("%s:%s: Cancel send for %p.",
-                            SRCNAME, __func__, m_object);
-                  wchar_t *title = utf8_to_wchar (_("GpgOL: Encryption not possible!"));
-                  wchar_t *msg = utf8_to_wchar (_(
-                  "Outlook returned an error when trying to send the encrypted mail.\n\n"
-                  "Please restart Outlook and try again.\n\n"
-                  "If it still fails consider using an encrypted attachment or\n"
-                  "switching to PGP/Inline in GpgOL's options."));
-                  MessageBoxW (get_active_hwnd(), msg, title,
-                               MB_ICONERROR | MB_OK);
-                  xfree (msg);
-                  xfree (title);
-                  *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
-                  return S_OK;
                 }
               log_debug ("%s:%s: Passing send event for message %p.",
                          SRCNAME, __func__, m_object);
@@ -444,6 +504,8 @@ EVENT_SINK_INVOKE(MailItemEvents)
               *(parms->rgvarg[0].pboolVal) = VARIANT_TRUE;
             }
 
+          log_debug ("%s:%s: Passing write event. Subject: %s Body: %s MsgCls: %s",
+                     SRCNAME, __func__, m_mail->get_subject().c_str(), m_mail->get_body().c_str(), get_oom_string (m_object, "MessageClass"));
           log_debug ("%s:%s: Passing write event.",
                      SRCNAME, __func__);
           m_mail->set_needs_save (false);
