@@ -95,6 +95,14 @@ sink_std_write (sink_t sink, const void *data, size_t datalen)
   return 0;
 }
 
+int
+sink_string_write (sink_t sink, const void *data, size_t datalen)
+{
+  Mail *mail = static_cast<Mail *>(sink->cb_data);
+  mail->append_to_inline_body (std::string((char*)data, datalen));
+  return 0;
+}
+
 /* Write method used with a sink_t that contains a file object.  */
 int
 sink_file_write (sink_t sink, const void *data, size_t datalen)
@@ -1139,7 +1147,7 @@ static int
 finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
                   protocol_t protocol, int encrypt, bool is_inline = false)
 {
-  HRESULT hr;
+  HRESULT hr = 0;
   SPropValue prop;
   SPropTagArray proparray;
 
@@ -1154,7 +1162,13 @@ finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
       prop.Value.lpszA = strdup ("IPM.Note.InfoPathForm.GpgOLS.SMIME.MultipartSigned");
     }
 
-  hr = message->SetProps(1, &prop, NULL);
+  if (!is_inline)
+    {
+      /* For inline we stick with IPM.Note because Exchange Online would
+         error out if we tried our S/MIME conversion trick with a text
+         plain message */
+      hr = message->SetProps(1, &prop, NULL);
+    }
   xfree(prop.Value.lpszA);
   if (hr)
     {
@@ -1366,17 +1380,19 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
     }
 
   /* Now the html body. It is somehow not accessible through PR_HTML,
-     OutlookSpy also shows MAPI Unsported (but shows the data) strange.
+     OutlookSpy also shows MAPI Unsupported (but shows the data) strange.
      We just cache it. Memory is cheap :-) */
-  const auto html_body = mail->get_cached_html_body();
-  if (html_body.empty())
+  char *html_body = mail->take_cached_html_body();
+  if (!html_body)
     {
       log_error ("%s:%s: BUG: Body but no html body in alternative mail?",
                  SRCNAME, __func__);
+      return -1;
     }
 
-  rc = write_part (sink, html_body.c_str(), html_body.size(),
+  rc = write_part (sink, html_body, strlen (html_body),
                    alt_boundary, NULL, 2);
+  xfree (html_body);
   if (rc)
     {
       TRACEPOINT;
@@ -1406,6 +1422,7 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
       if ((rc=write_multistring (sink,
                                  "Content-Type: multipart/related;\r\n",
                                  "\tboundary=\"", outer_boundary, "\"\r\n",
+                                 "\r\n", /* <--- Outlook adds an extra line. */
                                  NULL)))
         return rc;
     }
@@ -1415,6 +1432,7 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
       if ((rc=write_multistring (sink,
                                  "Content-Type: multipart/mixed;\r\n",
                                  "\tboundary=\"", outer_boundary, "\"\r\n",
+                                 "\r\n", /* <--- Outlook adds an extra line. */
                                  NULL)))
         return rc;
     }
@@ -1432,6 +1450,7 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
       if ((rc=write_multistring (sink,
                                  "Content-Type: multipart/related;\r\n",
                                  "\tboundary=\"", inner_boundary, "\"\r\n",
+                                 "\r\n", /* <--- Outlook adds an extra line. */
                                  NULL)))
         {
           return rc;
@@ -1521,8 +1540,18 @@ do_mime_sign (LPMESSAGE message, HWND hwnd, protocol_t protocol,
         return -1;
     }
 
+  /* Take the Body from the mail if possible. This is a fix for
+     GnuPG-Bug-ID: T3614 because the body is not always properly
+     updated in MAPI when sending. */
+  if (mail)
+    {
+      body = mail->take_cached_plain_body ();
+    }
   /* Get the attachment info and the body.  */
-  body = mapi_get_body (message, NULL);
+  if (!body)
+    {
+      body = mapi_get_body (message, NULL);
+    }
   if (body && !*body)
     {
       xfree (body);
@@ -1863,6 +1892,10 @@ create_top_encryption_header (sink_t sink, protocol_t protocol, char *boundary,
   if (is_inline)
     {
       *boundary = 0;
+      rc = 0;
+      /* This would be nice and worked for Google Sync but it failed
+         for Microsoft Exchange Online *sigh* so we put the body
+         instead into the oom body property and stick with IPM Note.
       rc = write_multistring (sink,
                               "MIME-Version: 1.0\r\n"
                               "Content-Type: text/plain;\r\n"
@@ -1870,6 +1903,7 @@ create_top_encryption_header (sink_t sink, protocol_t protocol, char *boundary,
                               "Content-Transfer-Encoding: 7BIT\r\n"
                               "\r\n",
                               NULL);
+     */
     }
   else if (protocol == PROTOCOL_SMIME)
     {
@@ -1927,7 +1961,7 @@ mime_encrypt (LPMESSAGE message, HWND hwnd,
 {
   int result = -1;
   int rc;
-  LPATTACH attach;
+  LPATTACH attach = nullptr;
   struct sink_s sinkmem;
   sink_t sink = &sinkmem;
   struct sink_s encsinkmem;
@@ -1943,17 +1977,25 @@ mime_encrypt (LPMESSAGE message, HWND hwnd,
   memset (sink, 0, sizeof *sink);
   memset (encsink, 0, sizeof *encsink);
 
-  attach = create_mapi_attachment (message, sink);
-  if (!attach)
-    return -1;
-
   /* Get the attachment info and the body.  We need to do this before
-     creating the engine's filter sue problem sending the cancel to
-     the engine with nothing for the engine to process.  This is
-     actually a bug in our engine code but we better avoid triggering
-     this bug because the engine sometimes hangs.  Fixme: Needs a
-     proper fix. */
-  body = mapi_get_body (message, NULL);
+     creating the engine's filter because sending the cancel to
+     the engine with nothing for the engine to process.  Will result
+     in an error. This is actually a bug in our engine code but
+     we better avoid triggering this bug because the engine
+     sometimes hangs.  Fixme: Needs a proper fix. */
+
+
+  /* Take the Body from the mail if possible. This is a fix for
+     GnuPG-Bug-ID: T3614 because the body is not always properly
+     updated in MAPI when sending. */
+  if (mail)
+    {
+      body = mail->take_cached_plain_body ();
+    }
+  if (!body)
+    {
+      body = mapi_get_body (message, NULL);
+    }
   if (body && !*body)
     {
       xfree (body);
@@ -1975,6 +2017,18 @@ mime_encrypt (LPMESSAGE message, HWND hwnd,
       is_inline = false;
     }
 
+  if (!is_inline || !mail)
+    {
+      attach = create_mapi_attachment (message, sink);
+      if (!attach)
+        return -1;
+    }
+  else
+    {
+      sink->cb_data = mail;
+      sink->writefnc = sink_string_write;
+    }
+
   /* Prepare the encryption.  We do this early as it is quite common
      that some recipient keys are not available and thus the
      encryption will fail early. */
@@ -2057,7 +2111,7 @@ mime_encrypt (LPMESSAGE message, HWND hwnd,
   if (*boundary && (rc = write_boundary (sink, boundary, 1)))
     goto failure;
 
-  if (close_mapi_attachment (&attach, sink))
+  if (attach && close_mapi_attachment (&attach, sink))
     goto failure;
 
   if (finalize_message (message, att_table, protocol, 1, is_inline))
@@ -2067,7 +2121,10 @@ mime_encrypt (LPMESSAGE message, HWND hwnd,
 
  failure:
   engine_cancel (filter);
-  cancel_mapi_attachment (&attach, sink);
+  if (attach)
+    {
+      cancel_mapi_attachment (&attach, sink);
+    }
   xfree (body);
   mapi_release_attach_table (att_table);
   xfree (my_sender);
