@@ -1442,9 +1442,9 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
       return NULL;
     }
 
-  char *ctx_name = get_object_name (context);
+  const auto ctx_name = std::string (get_object_name (context));
 
-  if (!ctx_name)
+  if (ctx_name.empty())
     {
       log_error ("%s:%s: Failed to get context name",
                  SRCNAME, __func__);
@@ -1452,13 +1452,30 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
       return NULL;
 
     }
-  if (!strcmp (ctx_name, "_Inspector"))
+  if (!strcmp (ctx_name.c_str(), "_Inspector"))
     {
       mailitem = get_oom_object (context, "CurrentItem");
     }
-  else if (!strcmp (ctx_name, "_Explorer"))
+  else if (!strcmp (ctx_name.c_str(), "_Explorer"))
     {
-      mailitem = get_oom_object (context, "Selection.Item(1)");
+      LPDISPATCH selection = get_oom_object (context, "Selection");
+      if (!selection)
+        {
+          log_error ("%s:%s: Failed to get selection.",
+                     SRCNAME, __func__);
+          gpgol_release (context);
+          return NULL;
+        }
+      int count = get_oom_int (selection, "Count");
+      if (count == 1)
+        {
+          // If we call this on a selection with more items
+          // Outlook sends an ItemLoad event for each mail
+          // in that selection.
+          mailitem = get_oom_object (selection, "Item(1)");
+        }
+      gpgol_release (selection);
+
       if (!mailitem)
         {
           *none_selected = true;
@@ -1468,6 +1485,14 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
           // Avoid showing wrong crypto state if we don't have a reading
           // pane. In that case the parser will finish for a mail which is gone
           // and the crypto state will not get updated.
+          //
+          //
+          // Somehow latest Outlook 2016 crashes when accessing the current view
+          // of the Explorer. This is even reproducible with
+          // GpgOL disabled and only with Outlook Spy active. If you select
+          // the explorer of an Outlook.com resource and then access
+          // the CurrentView and close the CurrentView again in Outlook Spy
+          // outlook crashes.
           LPDISPATCH prevEdit = get_oom_object (context, "PreviewPane.WordEditor");
           gpgol_release (prevEdit);
           if (!prevEdit)
@@ -1506,12 +1531,10 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
   gpgol_release (context);
   if (!mailitem)
     {
-      log_error ("%s:%s: Failed to get mailitem. From %s",
-                 SRCNAME, __func__, ctx_name);
-      xfree (ctx_name);
+      log_debug ("%s:%s: No mailitem. From %s",
+                 SRCNAME, __func__, ctx_name.c_str());
       return NULL;
     }
-  xfree (ctx_name);
 
   char *uid;
   /* Get the uid of this item. */
@@ -1622,7 +1645,7 @@ HRESULT get_sig_stip (LPDISPATCH ctrl, VARIANT *result)
       result->bstrVal = SysAllocString (L"");
       return S_OK;
     }
-  if (!mail || (!mail->is_signed () && !mail->is_encrypted ()))
+  if (!mail || !mail->is_crypto_mail ())
     {
       wchar_t *w_result;
       w_result = utf8_to_wchar (utf8_gettext ("You cannot be sure who sent, "
